/* Schedwi
   Copyright (C) 2007 Herve Quatremain

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Library General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/* schedwiping.c -- Test the connection with agents (schedwiclnt) */

#include <schedwi.h>

#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#if HAVE_UNISTD_H
#include <unistd.h>
#endif

#if HAVE_SYS_WAIT_H
# include <sys/wait.h>
#endif

#if HAVE_SIGNAL_H
#include <signal.h>
#endif

#if HAVE_GETOPT_H
#include <getopt.h>
#endif

#ifdef HAVE_LOCALE_H
#include <locale.h>
#endif

#if HAVE_ASSERT_H
#include <assert.h>
#endif

#include <sql_common.h>
#include <utils.h>
#include <conf.h>
#include <conf_srv.h>
#include <net_module_name.h>
#include <net_utils.h>
#include <net_utils_sock.h>
#include <sql_hosts.h>

static const char *configuration_file = NULL;


/*
 * Print a help message to stderr
 */
void
help (const char * prog_name)
{
	int i;
	char * msg[] = {
N_("Test the connection to a Schedwi agent (schedwiclnt)."),
N_("This program must be run on the same host as the Schedwi server."),
"",
#if HAVE_GETOPT_LONG
N_("  -c, --config=FILE  use the configuration file FILE rather than"),
N_("                     the default one"),
N_("  -h, --help         display this help and exit"),
N_("  -V, --version      print version information and exit"),
#else /* HAVE_GETOPT_LONG */
N_("  -c FILE            use the configuration file FILE rather than"),
N_("                     the default one"),
N_("  -h                 display this help and exit"),
N_("  -V                 print version information and exit"),
#endif /* ! HAVE_GETOPT_LONG */
"",
"~",
N_("Exit status is 1 on error (0 means no error)."),
NULL
	};

#if HAVE_ASSERT_H
	assert (prog_name != NULL && configuration_file != NULL);
#endif

	fprintf (stderr, _("Usage: %s [OPTION]... HOST\n"), prog_name);
						
	for (i = 0; msg[i] != NULL; i++) {
		if (msg[i][0] == '~') {
			fputs (	_("The default configuration file is "),
				stderr);
			fputs (configuration_file, stderr);
		}
		else {
			fputs ((msg[i][0] == '\0')? msg[i]: _(msg[i]), stderr);
		}
		fputc ('\n', stderr);
	}

	fputc ('\n', stderr);
	fputs (_("Report bugs to "), stderr);
	fputs (PACKAGE_BUGREPORT, stderr);
	fputc ('\n', stderr);
}


/*
 * Error callback function for the sql_host_get_main_row_names() function
 */
static void
sql_error_logger (void *data, const char *msg, unsigned int err_code)
{
	if (msg != NULL) {
		fputs (msg, stderr);
		fputc ('\n', stderr);
	}
	else {
		fputs (
		_("Database error while retrieving the client host details\n"),
			stderr);
	}
}


/*
 * Main function
 *
 * The exit code is 1 in case of error or 0 if killed (SIGTERM)
 */
int
main (int argc, char **argv)
{
	const char *server_key, *server_crt;
	const char *prog_name;
	char *agent_name, *module, *result_msg;
	unsigned int result_msg_len;
	char **names;
	char **host_details;
	unsigned long int *host_details_len;
	int ret, module_len, ret_read;
	MYSQL *sql_handler;
	char *err_msg;
	schedwi_BIO *b;

#if HAVE_GETOPT_LONG
	int option_index;
	struct option long_options[] =
	{
		{"help",       0, 0, 'h'},
		{"version",    0, 0, 'V'},
		{"config",     1, 0, 'c'},
		{0, 0, 0, 0}
	};
#endif

#if HAVE_SETLOCALE
	setlocale (LC_ALL, "");
#endif
#if HAVE_BINDTEXTDOMAIN
	bindtextdomain (PACKAGE, LOCALEDIR);
#endif
#if HAVE_TEXTDOMAIN
	textdomain (PACKAGE);
#endif


	/* Set default values for options */
	configuration_file = SCHEDWI_DEFAULT_CONFFILE_SRV;

	prog_name = base_name (argv[0]);

	/* Parse options */
	while (1) {
#if HAVE_GETOPT_LONG
		option_index = 0;
		ret = getopt_long (argc, argv, "hVc:",
					long_options, &option_index);
#else
		ret = getopt (argc, argv, "hVc:");
#endif

		if (ret == -1) {
			break;
		}

		switch (ret) {
			case 'h':
				help (prog_name);
				return 0;
			case 'V':
				version (prog_name);
				return 0;
			case 'c':
				configuration_file = optarg;
				break;
			default:
				help (prog_name);
				return 1;
		}
	}

	/* Agent host name missing */
	if (optind >= argc) {
		fputs (_("Agent host name required\n"), stderr);
		help (prog_name);
		return 1;
	}

	/* Too many parameters */
	if (optind + 1 != argc) {
		fputs (_("Too many parameters\n"), stderr);
		help (prog_name);
		return 1;
	}

	agent_name = argv[optind];

	/*
	 * Read the configuration file
	 */
	ret = conf_init_srv (configuration_file);
	switch (ret) {
		case -1:
			fputs (_("Memory allocation error\n"), stderr);
			break;

		case -2:
			perror (configuration_file);
			break;
	}
	if (ret != 0) {
		return 1;
	}

	/*
	 * Initialize the network
	 */
	ret = conf_get_param_string ("SSLServerCertificateFile", &server_crt);
	ret += conf_get_param_string (	"SSLServerCertificateKeyFile",
					&server_key);
#if HAVE_ASSERT_H
	assert (ret == 0);
#endif

	if (net_init (server_crt, server_key) != 0) {
		conf_destroy_srv ();
		return 1;
	}

	/*
	 * Database connection
	 */
	err_msg = NULL;
	sql_handler = begin_mysql (&err_msg);
	if (sql_handler == NULL) {
		if (err_msg != NULL) {
			fprintf (	stderr,
					_("Failed to connect to database: %s\n"),
					err_msg);
			free (err_msg);
		}
		else {
			fputs (_("Failed to connect to database\n"), stderr);
		}
		conf_destroy_srv ();
		net_destroy ();
		return 1;
	}

	/* Retrieve all the names of the agent from the DNS */
	if (build_client_names_from_name (agent_name, &names) != 0) {
		end_mysql ();
		conf_destroy_srv ();
		net_destroy ();
		return 1;
	}

	/* Retrieve the agent details from the database */
	if (sql_host_get_main_row_names (names, 
					&host_details, &host_details_len,
					sql_error_logger, NULL) != 0)
	{
		destroy_client_names (names);
		end_mysql ();
		conf_destroy_srv ();
		net_destroy ();
		return 1;
	}
	destroy_client_names (names);
	end_mysql ();

	/* Compose the module name */
	module = NULL;
	module_len = build_module_name ("version", &module);
	if (module_len < 0) {
		sql_free_row (host_details);
		free (host_details_len);
		conf_destroy_srv ();
		net_destroy ();
		fputs (_("Memory allocation error\n"), stderr);
		return 1;
	}

	/* Establish the connection to the client */
	b = net_client (host_details[2],  /* TCP port */
			host_details[1],  /* Client host name */
			(host_details[3][0] == '0') ? 0: 1, /* SSL */
			host_details[4],  /* Certificate */
			(unsigned int)(host_details_len[4]));
	free (host_details_len);
	sql_free_row (host_details);
	if (b == NULL) {
		free (module);
		conf_destroy_srv ();
		net_destroy ();
		return 1;
	}

	if (	   net_write (b, module, (int)module_len) != 0
		|| net_write_eof (b) != 0)
	{
		net_close (b);
		free (module);
		conf_destroy_srv ();
		net_destroy ();
		return 1;
	}
	free (module);

	/* Read the result */
	result_msg = NULL;
	result_msg_len = 0;
	ret_read = net_read (b, &result_msg, &result_msg_len);
	net_close (b);

	/* System error */
	if (ret_read < 0) {
		if (result_msg != NULL) {
			free (result_msg);
		}
		conf_destroy_srv ();
		net_destroy ();
		return 1;
	}

	/* Success */
	if (result_msg != NULL) {
		result_msg[ret_read] = '\0';
		printf (_("The agent v%s is running.\n"), result_msg);
		free (result_msg);
	}
	else {
		fputs (_("The agent is running.\n"), stdout);
	}

	/* Destroy the configuration file object */
	conf_destroy_srv ();

	/* Clean the network layer */
	net_destroy ();

	return 0;
}

/*------------------------======= End Of File =======------------------------*/
